@@ -19,7 +19,7 @@
 ## You should have received a copy of the GNU General Public License
 ## along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import json, os, gzip, math, re
+import json, os, math, re, sys
 from datetime import datetime
 
 try:
@@ -32,6 +32,8 @@ try:
 except ImportError:
     from .ordereddict import OrderedDict
 
+from .util import gzip_open
+
 # Controls pretty-printing of json dumps
 JSON_INDENT=None
 
@@ -86,11 +88,15 @@ class ResultSet(object):
             raise RuntimeError("Unexpected data point(s): %s" % list(data.keys()))
 
     def last_datapoint(self, series):
-        if not self._results[series]:
+        data = self.series(series)
+        if not data:
             return None
-        return self._results[series][-1]
+        return data[-1]
 
     def series(self, name, smooth=None):
+        if not name in self._results:
+            sys.stderr.write("Warning: Missing data points for series '%s'\n" % name)
+            return [None]*len(self.x_values)
         if smooth:
             return self.smoothed(name, smooth)
         return self._results[name]
@@ -139,7 +145,11 @@ class ResultSet(object):
         return not self._x_values
 
     def dump(self, fp):
-        return json.dump(self.serialise(), fp, indent=JSON_INDENT)
+        data = self.dumps()
+        if hasattr(data, "decode"):
+            data = data.decode()
+
+        return fp.write(data)
 
     def dumps(self):
         return json.dumps(self.serialise(), indent=JSON_INDENT)
@@ -161,10 +171,14 @@ class ResultSet(object):
     def dump_dir(self, dirname):
         self._dump_file = os.path.join(dirname, self._gen_filename())
         try:
-            fp = gzip.open(self._dump_file, "wt")
-            self.dump(fp)
-        finally:
-            fp.close()
+            fp = gzip_open(self._dump_file, "wt")
+            try:
+                self.dump(fp)
+            finally:
+                fp.close()
+        except IOError as e:
+            sys.stderr.write("Unable to write results data file: %s\n" % e)
+            self._dump_file = None
 
     @classmethod
     def unserialise(cls, obj):
@@ -188,7 +202,7 @@ class ResultSet(object):
     def load_file(cls, filename):
         try:
             if filename.endswith(".gz"):
-                o = gzip.open
+                o = gzip_open
             else:
                 o = open
             fp = o(filename, 'rt')
